#!/usr/bin/env python
# -*- coding: utf-8 -*-

# script to get share data from multiple web sites for comparison
# rev 0.0 21Dec13
# rev 0.1 21Dec13 Added some error traps and cleaned up currency in DL dividends
# rev 0.2 22Dec13 Added data from iii and more cleanup
# rev 0.3 23Dec13 Added more data from iii and prices from MSN money
# rev 0.4 23Dec13 Fixed problem with morningstar dividends
# rev 0.5 26Dec13 Message Box now works in LO and OO, but still problems with htmlparser in OO
# rev 0.6 26Dec13 Fixed issues with OO. Solved issue of htmlparser, and resolved issue with £ symbol and replace function in
#                 Digital Look data
# rev 0.7 26Dec13 Added Parameters sheet and read webTimeOut
# rev 0.8 27Dec13 Added try/except for HTML parsing
# rev 0.81 27Dec13 Added trap for non-numeric values
# rev 0.82 27Dec13 Added error message for urllib
# rev 0.83 28Dec13 Added if/exists test for all data written to sheet
# rev 0.84 30Dec13 Fixed problem with non-numeric dividends in ADVFN
#                  Added Clear for company website link
# rev 0.85 04Jan14 Added special dividend marker for digital look
# rev 0.86 04Jan14 Added correct interpretation of cent symbol in digital look
# rev 0.87 05Jan14 Added special dividend marker for ADVFN
# rev 0.88 05Jan14 Added special dividend marker for MoneyAM
# rev 0.89 05Jan14 Added special dividend marker for Interactive Investor
# rev 0.9  06Jan14 Added special dividend marker for Dividend Investor
# rev 1.00 08Jan14 Added check for current version, published to Weebly and announced to TMF
# rev 1.10 30Apr14 Fixed problem with iii 
#						they have redesigned the web site, with missing data, mix of Epic and 
#						Yahoo symbol format


import re
import datetime
from com.sun.star.awt import WindowDescriptor
import sys
import traceback

if sys.version[0:1] == '3':
	import urllib.request
	from html.parser import HTMLParser
else:
	import urllib2
	from HTMLParser import HTMLParser

### define function for message box
def messageBox(msgText, msgTitle):
	doc = XSCRIPTCONTEXT.getDocument()
	parentwin = doc.CurrentController.Frame.ContainerWindow

	try:
		messageBoxLO4(parentwin, msgText, msgTitle)
	except:
		messageBoxOO4(parentwin, msgText, msgTitle)

### following works with OO4 but not LO4 or OO Portable 3.2
def messageBoxOO4(ParentWin, MsgText, MsgTitle):
	#doc = XSCRIPTCONTEXT.getDocument()
	ctx = XSCRIPTCONTEXT.getComponentContext()
	MsgText = MsgText
	
	toolkit = ctx.getServiceManager().createInstanceWithContext("com.sun.star.awt.Toolkit", ctx)
	msgbox = toolkit.createMessageBox(ParentWin, 0, 1, MsgTitle, MsgText)
    
	msgbox.execute()
	#msgbox.dispose()

# Show a message box with the UNO based toolkit. Works with LO4 and OOP3.2
def messageBoxLO4(ParentWin, MsgText, MsgTitle):
	MsgText = MsgText
	#describe window properties.
	aDescriptor = WindowDescriptor()
	aDescriptor.Type = 1   					# MODALTOP
	aDescriptor.WindowServiceName = "infobox"
	aDescriptor.ParentIndex = -1
	aDescriptor.Parent = ParentWin
	aDescriptor.WindowAttributes = 4194304	#MsgButton = OK
	
	tk = ParentWin.getToolkit()
	msgbox = tk.createWindow(aDescriptor)
	
	msgbox.setMessageText(MsgText)
	msgbox.setCaptionText(MsgTitle)
		
	return msgbox.execute()



class htmlParser1(HTMLParser):
# ADVFN data
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.marketCapFound = False
		self.marketCap = ''
		self.sharesInIssueFound = False
		self.sharesInIssue = ''
		self.peFound = False
		self.pe = ''
		self.wwwAddressFound = False
		self.wwwAddress = ''
		self.priceFound = False
		self.keyFiguresFound = False
		self.dividendYieldFound = False
		self.dividendYield = ''
		self.dividendCoverFound = False
		self.dividendCover = ''
		self.grossGearingFound = False
		self.grossGearing = ''
		self.divTableFound = False
		self.divRowFound = False
		self.divList = []
		self.divYrp1 = 0
		self.divYr0 = 0
		self.divYrm1 = 0
		self.divYrm2 = 0
		self.divYrm3 = 0
		self.divYrm4 = 0
		self.divCurYrp1 = ''
		self.divCurYr0 = ''
		self.divCurYrm1 = ''
		self.divCurYrm2 = ''
		self.divCurYrm3 = ''
		self.divCurYrm4 = ''
		self.payYear = ''
		self.divSpecYrp1 = False
		self.divSpecYr0 = False
		self.divSpecYrm1 = False
		self.divSpecYrm2 = False
		self.divSpecYrm3 = False
		self.divSpecYrm4 = False
		self.priceList = []
		self.lastPrice = 0
		self.bidPrice = 0
		self.offerPrice = 0
		self.sectorFound = False
		self.sector = ''
		

	def handle_starttag(self, tag, attrs):
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True

	def handle_data(self, data):
		if self.wwwAddressFound:
			self.wwwAddress = data
			self.wwwAddressFound = False
		if data == 'WWW Address:':
			self.wwwAddressFound = True
			
		if self.sectorFound:
			self.sector = data
			self.sectorFound = False
		if data == 'Industry Sector:':
			self.sectorFound = True

		if self.priceFound:
			data = data.strip()
			if data:
				self.priceList.append(data)
		if data == 'Price':
			self.priceFound = True
		if data == 'Volume' and self.priceFound:
			self.priceList = []
		if data.startswith('Market Cap'):
			self.priceFound = False
			
		if self.marketCapFound and self.keyFiguresFound:
			self.marketCap = self.marketCap + data
		if data == 'Market Cap.':
			self.marketCapFound = True
			
		if self.sharesInIssueFound and self.keyFiguresFound:
			self.sharesInIssue = self.sharesInIssue + data
		if data == 'Shares In Issue':
			self.sharesInIssueFound = True
			
		if self.peFound and self.keyFiguresFound:
			self.pe = data
		if data == 'PE Ratio':
			self.peFound = True
		
		if self.dividendYieldFound and self.keyFiguresFound:
			self.dividendYield = self.dividendYield + data
		if data == 'Dividend Yield':
			self.dividendYieldFound = True

		if self.dividendCoverFound and self.keyFiguresFound:
			self.dividendCover = data
		if data == 'Dividend Cover':
			self.dividendCoverFound = True
			
		if self.grossGearingFound and self.keyFiguresFound:
			self.grossGearing = self.grossGearing + data
		if data == 'Gross Gearing':
			self.grossGearingFound = True
		
		if data.endswith('Key Figures'):
			self.keyFiguresFound = True
		
		if data.endswith('Announcement Date'):
			self.divTableFound = True
		matchObj = re.search( '\d\d \w\w\w \d\d', data)
		if matchObj:
			self.divRowFound = True
		if self.divRowFound:
			self.divList.append(data)
			if len(self.divList) == 9 and not self.divList[3] == '-':
				#print(self.divList)
				self.payYear = self.divList[8]
				if len(self.payYear) == 10:
					self.payYear = self.payYear[-4:]
					if self.payYear == str(yrp1):
						self.divYrp1 = self.divYrp1 + float(self.divList[3])
						self.divCurYrp1 = self.divList[2]
						if 'Special' in self.divList[1]: self.divSpecYrp1 = True
					elif self.payYear == str(yr0):
						self.divYr0 = self.divYr0 + float(self.divList[3])
						self.divCurYr0 = self.divList[2]
						if 'Special' in self.divList[1]: self.divSpecYr0 = True
						#print('--------', yr0, self.divYr0)
					elif self.payYear == str(yrm1):
						self.divYrm1 = self.divYrm1 + float(self.divList[3])
						self.divCurYrm1 = self.divList[2]
						if 'Special' in self.divList[1]: self.divSpecYrm1 = True
					elif self.payYear == str(yrm2):
						self.divYrm2 = self.divYrm2 + float(self.divList[3])
						self.divCurYrm2 = self.divList[2]
						if 'Special' in self.divList[1]: self.divSpecYrm2 = True
					elif self.payYear == str(yrm3):
						self.divYrm3 = self.divYrm3 + float(self.divList[3])
						self.divCurYrm3 = self.divList[2]
						if 'Special' in self.divList[1]: self.divSpecYrm3 = True
					elif self.payYear == str(yrm4):
						self.divYrm4 = self.divYrm4 + float(self.divList[3])
						self.divCurYrm4 = self.divList[2]
						if 'Special' in self.divList[1]: self.divSpecYrm4 = True
		
			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			if self.marketCapFound:
				self.marketCapFound = False
			if self.sharesInIssueFound:
				self.sharesInIssueFound = False
			if self.peFound:
				self.peFound = False
			if self.dividendYieldFound:
				self.dividendYieldFound = False
			if self.dividendCoverFound:
				self.dividendCoverFound = False
			if self.grossGearingFound:
				self.grossGearingFound = False
			if self.divRowFound:
				self.divRowFound = False
				self.divList = []
			
			
		if tag == 'td':
			self.in_td = False
		
		if tag == 'table' and self.keyFiguresFound:
			self.keyFiguresFound = False
		if tag == 'table' and self.divTableFound:
			self.divTableFound = False
			
### end of htmlParser1 to get data from ADVFN
#######################################################

######################################################
### MoneyAM data
class htmlParser2(HTMLParser):
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.marketCapFound = False
		self.marketCap = ''
		self.sharesInIssueFound = False
		self.sharesInIssue = ''
		self.peFound = False
		self.pe = ''
		self.dividendYieldFound = False
		self.dividendYield = ''
		self.dividendCoverFound = False
		self.dividendCover = ''
		self.grossGearingFound = False
		self.grossGearing = ''
		self.dividendTableFound = False
		self.dividendRowFound = False
		self.divList = []
		self.divYrp1 = 0
		self.divYr0 = 0
		self.divYrm1 = 0
		self.divYrm2 = 0
		self.divYrm3 = 0
		self.divYrm4 = 0
		self.divCurYrp1 = ''
		self.divCurYr0 = ''
		self.divCurYrm1 = ''
		self.divCurYrm2 = ''
		self.divCurYrm3 = ''
		self.divCurYrm4 = ''
		self.divSpecYrp1 = False
		self.divSpecYr0 = False
		self.divSpecYrm1 = False
		self.divSpecYrm2 = False
		self.divSpecYrm3 = False
		self.divSpecYrm4 = False
		self.payYear = ''
		self.latestPrice = ''
		self.bidPrice = ''
		self.offerPrice = ''
		self.latestPriceFound = False
		self.bidPriceFound = False
		self.offerPriceFound = False


	def handle_starttag(self, tag, attrs):
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
		
		if len(attrs) == 2:
			if attrs[0][1] == 'current-price':
				self.latestPriceFound = True
		if len(attrs) == 1:
			if attrs[0][1] == 'bid-price':
				self.bidPriceFound = True
			elif attrs[0][1] == 'ask-price':
				self.offerPriceFound = True

	def handle_data(self, data):

		if self.latestPriceFound:
			self.latestPrice = data
			self.latestPriceFound = False
		if self.bidPriceFound:
			self.bidPrice = data
			self.bidPriceFound = False
		if self.offerPriceFound:
			self.offerPrice = data
			self.offerPriceFound = False
			
		if self.marketCapFound:
			self.marketCap = data
		if data.startswith('Market Cap'):
			self.marketCapFound = True
			
		if self.sharesInIssueFound:
			self.sharesInIssue = data
		if data.startswith('Shares in issue'):
			self.sharesInIssueFound = True
			
		if self.peFound:
			self.pe = data
		if data == 'P/E Ratio':
			self.peFound = True
		
		if self.dividendYieldFound:
			self.dividendYield = data
		if data.startswith('Dividend Yield'):
			self.dividendYieldFound = True

		if self.dividendCoverFound:
			self.dividendCover = data
		if data.startswith('Dividend cover'):
			self.dividendCoverFound = True
			
		if self.grossGearingFound:
			self.grossGearing = data
		if data.startswith('Gross Gearing'):
			self.grossGearingFound = True
		
		if data.endswith('Dividend History'):
			self.dividendTableFound = True
			
		if self.dividendTableFound:
			if data.endswith('Dividend'):
				self.dividendRowFound = True
		if self.dividendTableFound and self.dividendRowFound:
			data = data.strip()
			if data:
				self.divList.append(data)
			if len(self.divList) == 6:
				if self.divList[2].endswith(str(yrp1)):
					self.divYrp1 = self.divYrp1 + float(self.divList[3])
					self.divCurYrp1 = self.divList[4]
					if 'Special' in self.divList[0]: self.divSpecYrp1 = True
				elif self.divList[2].endswith(str(yr0)):
					self.divYr0 = self.divYr0 + float(self.divList[3])
					self.divCurYr0 = self.divList[4]
					if 'Special' in self.divList[0]: self.divSpecYr0 = True
				elif self.divList[2].endswith(str(yrm1)):
					self.divYrm1 = self.divYrm1 + float(self.divList[3])
					self.divCurYrm1 = self.divList[4]
					if 'Special' in self.divList[0]: self.divSpecYrm1 = True
				elif self.divList[2].endswith(str(yrm2)):
					self.divYrm2 = self.divYrm2 + float(self.divList[3])
					self.divCurYrm2 = self.divList[4]
					if 'Special' in self.divList[0]: self.divSpecYrm2 = True
				elif self.divList[2].endswith(str(yrm3)):
					self.divYrm3 = self.divYrm3 + float(self.divList[3])
					self.divCurYrm3 = self.divList[4]
					if 'Special' in self.divList[0]: self.divSpecYrm3 = True
				elif self.divList[2].endswith(str(yrm4)):
					self.divYrm4 = self.divYrm4 + float(self.divList[3])
					self.divCurYrm4 = self.divList[4]
					if 'Special' in self.divList[0]: self.divSpecYrm4 = True

					
				self.divList = []
			
			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			self.divList = []
			
		if tag == 'td':
			self.in_td = False
			if self.marketCapFound:
				self.marketCapFound = False
			if self.sharesInIssueFound:
				self.sharesInIssueFound = False
			if self.grossGearingFound:
				self.grossGearingFound = False	
			if self.peFound:
				self.peFound = False
			if self.dividendYieldFound:
				self.dividendYieldFound = False
			if self.dividendCoverFound:
				self.dividendCoverFound = False
				
		if tag == 'table':
			self.dividendTableFound = False
				

### end of htmlParser2 to parse data from MoneyAM
#################################################


######################################################
### Yahoo data
class htmlParser3(HTMLParser):
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.marketCapFound = False
		self.marketCap = ''
		self.latestPriceFound = False
		self.latestPrice = ''
		self.bidPriceFound = False
		self.bidPrice = ''
		self.offerPriceFound = False
		self.offerPrice = ''


	def handle_starttag(self, tag, attrs):
		#print(tag, attrs)
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
			
		if len(attrs) == 1:
			if attrs[0][1] == 'time_rtq_ticker':
				self.latestPriceFound = True

	def handle_data(self, data):
		if self.latestPriceFound:
			self.latestPrice = data
			self.latestPriceFound = False
		if self.bidPriceFound:
			self.bidPrice = data
			self.bidPriceFound = False
		if self.offerPriceFound:
			self.offerPrice = data
			self.offerPriceFound = False
			
		if data == 'Bid:':
			self.bidPriceFound = True
		if data == 'Ask:':
			self.offerPriceFound = True
			
		if self.marketCapFound:
			self.marketCap = data.strip()
		if data.startswith('Market Cap'):
			self.marketCapFound = True
		
			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			if self.marketCapFound:
				self.marketCapFound = False
			
		if tag == 'td':
			self.in_td = False
			

### end of htmlParser3 to parse data from Yahoo
#################################################

######################################################
### Morningstar data
class htmlParser4(HTMLParser):
	global yrp2
	global yrp1
	global yr0
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.marketCapFound = False
		self.marketCap = ''
		self.sharesInIssueFound = False
		self.sharesInIssue = ''
		self.peFound = False
		self.pe = ''
		self.dividendYieldFound = False
		self.dividendYield = ''
		self.dividendCoverFound = False
		self.dividendCover = ''
		self.grossGearingFound = False
		self.grossGearing = ''
		self.brokerForecastsFound = False
		self.yearEndingFound = False
		self.divList = []
		self.divFcstYrp2 = 0
		self.divFcstYrp1 = 0
		self.divFcstYr0 = 0
		self.latestPriceFound = False
		self.latestPrice = ''
		self.bidPriceFound = False
		self.bidPrice = ''
		self.offerPriceFound = False
		self.offerPrice = ''


	def handle_starttag(self, tag, attrs):
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True

			if len(attrs) == 2 and attrs[1][1] == 'Col0MCap':
				self.marketCapFound = True
			if len(attrs) == 2 and attrs[1][1] == 'Col0Yield':
				self.dividendYieldFound = True
			if len(attrs) == 2 and attrs[1][1] == 'Col0PE':
				self.peFound = True
				
		if len(attrs) == 1:
			if attrs[0][1] == 'Col0IntradayBid':
				self.bidPriceFound = True
		
		if len(attrs) == 1:
			if attrs[0][1] == 'Col0IntradayAsk':
				self.offerPriceFound = True
					

	def handle_data(self, data):
		if self.marketCapFound:
			self.marketCap = data
		if self.peFound:
			self.pe = data
		if self.dividendYieldFound:
			self.dividendYield = data
		if data == 'Broker Forecasts':
			self.brokerForecastsFound = True
		if data == 'Year Ending':
			self.yearEndingFound = True
		
		if self.latestPriceFound:
			self.latestPrice = data
			self.latestPriceFound = False
		if data == 'Last Price':
			self.latestPriceFound = True
			
		if self.bidPriceFound:
			self.bidPrice = data
			self.bidPriceFound = False
		if self.offerPriceFound:
			self.offerPrice = data
			self.offerPriceFound = False

			
		if self.brokerForecastsFound and self.yearEndingFound:
			#print(data)
			self.divList.append(data)
			if len(self.divList) == 8:
				if self.divList[0] == str(yrp2):
					self.divFcstYrp2 = self.divList[6]
				elif self.divList[0] == str(yrp1):
					self.divFcstYrp1 = self.divList[6]
				elif self.divList[0] == str(yr0):
					self.divFcstYr0 = self.divList[6]
			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			self.divList = []
			
			
		if tag == 'td':
			self.in_td = False
			self.marketCapFound = False
			self.dividendYieldFound = False
			self.peFound = False
		
		if tag == 'table':
			self.brokerForecastsFound = False
			self.yearEndingFound = False

### end of htmlParser4 to parse data from Morningstar
#################################################


#################################################
### htmlParser4a to parse dividends from Morningstar
class htmlParser4a(HTMLParser):
	global yrp2
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.incomeStatementFound = False
		self.divList = []
		self.divYr0 = 0
		self.y1Found = False
		self.y2Found = False
		self.y3Found = False
		self.y4Found = False
		self.y5Found = False
		self.dpsFound = False
		self.dpsY1 = 0
		self.dpsY2 = 0
		self.dpsY3 = 0
		self.dpsY4 = 0
		self.dpsY5 = 0
		self.y1 = 0
		self.y2 = 0
		self.y3 = 0
		self.y4 = 0
		self.y5 = 0
		self.curAttr = ''


	def handle_starttag(self, tag, attrs):
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
		
		if len(attrs) == 3:
			if attrs[1][1] == 'MsStockReportFisshDp':
				self.incomeStatementFound = True

		if len(attrs) == 2:
			if attrs[1][1] == 'MsStockReportFisshY1':
				self.y1Found = True
			elif attrs[1][1] == 'MsStockReportFisshY2':
				self.y2Found = True
			elif attrs[1][1] == 'MsStockReportFisshY3':
				self.y3Found = True
			elif attrs[1][1] == 'MsStockReportFisshY4':
				self.y4Found = True
			elif attrs[1][1] == 'MsStockReportFisshY5':
				self.y5Found = True
		self.curAttr = attrs

	def handle_data(self, data):
		#if self.incomeStatementFound:
			#print(data)
		if self.y1Found:
			self.y1 = data
			self.y1Found = False
		elif self.y2Found:
			self.y2 = data
			self.y2Found = False
		elif self.y3Found:
			self.y3 = data
			self.y3Found = False
		elif self.y4Found:
			self.y4 = data
			self.y4Found = False
		elif self.y5Found:
			self.y5 = data
			self.y5Found = False
				
		if self.incomeStatementFound and self.dpsFound:
			if self.curAttr[0][1] == 'MsStockReportFdshY1':
				self.dpsY1 = data
			elif self.curAttr[0][1] == 'MsStockReportFdshY2':
				self.dpsY2 = data
			elif self.curAttr[0][1] == 'MsStockReportFdshY3':
				self.dpsY3 = data
			elif self.curAttr[0][1] == 'MsStockReportFdshY4':
				self.dpsY4 = data
			elif self.curAttr[0][1] == 'MsStockReportFdshY5':
				self.dpsY5 = data

		if data == 'DPS':
			self.dpsFound = True

		
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			self.divList = []
			if self.dpsFound:
				self.y1Found = False
				self.y2Found = False
				self.y3Found = False
				self.y4Found = False
				self.y5Found = False
				self.dpsFound = False
				self.incomeStatementFound = False
			
		if tag == 'td':
			self.in_td = False
			
### end of htmlParser4a to parse data from Morningstar
#################################################


######################################################
### MSN Money data
class htmlParser5(HTMLParser):
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.marketCapFound = False
		self.marketCap = ''
		self.sharesInIssueFound = False
		self.sharesInIssue = ''
		self.peFound = False
		self.pe = ''
		self.dividendYieldFound = False
		self.dividendYield = ''
		self.dividendCoverFound = False
		self.dividendCover = ''
		self.grossGearingFound = False
		self.grossGearing = ''
		self.bidPrice = ''
		self.offerPrice = ''
		self.latestPrice = ''
		self.bidPriceFound = False
		self.offerPriceFound = False
		self.latestPriceFound = False
		self.quickquoteFound = False
		self.dataCount = 0


	def handle_starttag(self, tag, attrs):
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
		
		if len(attrs) == 2:
			if attrs[1][1] == 'quickquote':
				self.quickquoteFound = True

	def handle_data(self, data):
		data = data.strip()
		
		if self.marketCapFound:
			self.marketCap = data
		if data == 'Market Cap':
			self.marketCapFound = True
			
		if self.sharesInIssueFound:
			self.sharesInIssue = data
		if data == 'Shares Outstanding':
			self.sharesInIssueFound = True
			
		if self.peFound:
			self.pe = data
		if data == 'P/E':
			self.peFound = True

		if self.dividendYieldFound:
			self.dividendYield = data
		if data == 'Dividend Yield %':
			self.dividendYieldFound = True

		if data and self.quickquoteFound:
			self.dataCount += 1
		
		if self.dataCount == 2 and self.quickquoteFound:
			self.latestPrice = data
			self.quickquoteFound = False
			
		if self.bidPriceFound:
			self.bidPrice = data
			self.bidPriceFound = False
		if data == 'Bid':
			self.bidPriceFound = True
			
		if self.offerPriceFound:
			self.offerPrice = data
			self.offerPriceFound = False
		if data == 'Ask':
			self.offerPriceFound = True

			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			self.marketCapFound = False
			self.sharesInIssueFound = False
			self.dividendYieldFound = False
			self.peFound = False
			
		if tag == 'td':
			self.in_td = False
			
### end of htmlParser5 to parse data from MSN Money
#################################################

#######################################################
### htmlParser6 to parse dividends from Dividend Investor
class htmlParser6(HTMLParser):
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		
		self.divYrp1 = 0
		self.divYr0 = 0
		self.divYrm1 = 0
		self.divYrm2 = 0
		self.divYrm3 = 0
		self.divYrm4 = 0
		self.divSpecYrp1 = False
		self.divSpecYr0 = False
		self.divSpecYrm1 = False
		self.divSpecYrm2 = False
		self.divSpecYrm3 = False
		self.divSpecYrm4 = False
		self.divList = []
		self.lastPayDate = ''


	def handle_starttag(self, tag, attrs):
		if tag == 'tr' :
			self.in_tr = True
			self.divList = []
			
		if tag == 'td' :
			self.in_td = True
			

	def handle_data(self, data):
		data = data.strip()
		if data:
			self.divList.append(data.strip())
			if len(self.divList) == 6:
				if self.divList[4].endswith(str(yrp1)):
					self.divYrp1 = self.divYrp1 + float(self.divList[5])
					if self.divList[4] == self.lastPayDate: self.divSpecYrp1 = True
				elif self.divList[4].endswith(str(yr0)):
					self.divYr0 = self.divYr0 + float(self.divList[5])
					if self.divList[4] == self.lastPayDate: self.divSpecYr0 = True
				elif self.divList[4].endswith(str(yrm1)):
					self.divYrm1 = self.divYrm1 + float(self.divList[5])
					if self.divList[4] == self.lastPayDate: self.divSpecYrm1 = True
				elif self.divList[4].endswith(str(yrm2)):
					self.divYrm2 = self.divYrm2 + float(self.divList[5])
					if self.divList[4] == self.lastPayDate: self.divSpecYrm2 = True
				elif self.divList[4].endswith(str(yrm3)):
					self.divYrm3 = self.divYrm3 + float(self.divList[5])
					if self.divList[4] == self.lastPayDate: self.divSpecYrm3 = True
				elif self.divList[4].endswith(str(yrm4)):
					self.divYrm4 = self.divYrm4 + float(self.divList[5])
					if self.divList[4] == self.lastPayDate: self.divSpecYrm4 = True
				self.lastPayDate = self.divList[4]
			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			
		if tag == 'td':
			self.in_td = False
### end of htmlParser6 to parse dividends from Dividend Investor
#######################################################


#######################################################
### get dividend currency from dividend investor
class htmlParser6a(HTMLParser):
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.curFound = False
		self.marketCapFound = False
		self.sharesInIssueFound = False
		self.dividendYieldFound = False
		self.latestPriceFound = False
		self.divCur = ''
		self.marketCap = ''
		self.sharesInIssue = ''
		self.dividendYield = ''
		self.latestPrice = ''

	def handle_starttag(self, tag, attrs):
		if tag == 'tr' :
			self.in_tr = True

		if tag == 'td' :
			self.in_td = True

	def handle_data(self, data):
		if self.curFound:
			data = data.strip()
			if data: 
				self.divCur = data
				
		if self.marketCapFound:
			data = data.strip()
			if data:
				self.marketCap = data
				
		if self.sharesInIssueFound:
			data = data.strip()
			if data:
				self.sharesInIssue = data
			
		if self.dividendYieldFound:
			data = data.strip()
			if data:
				self.dividendYield = data
				
		if self.latestPriceFound:
			data = data.strip()
			if data:
				self.latestPrice = data
				
		if data == 'Currency: ':
			self.curFound = True
		if data == 'Market Cap: ':
			self.marketCapFound = True
		if data.startswith('Shares Outstanding'):
			self.sharesInIssueFound = True
		if data.startswith('Dividend Yield'):
			self.dividendYieldFound = True
		if data.startswith('Latest Price'):
			self.latestPriceFound = True

			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			self.curFound = False
			self.marketCapFound = False
			self.sharesInIssueFound = False
			self.dividendYieldFound = False
			self.latestPriceFound = False
			
		if tag == 'td':
			self.in_td = False
			
### end of htmlParser6a
#######################################################


#######################################################
### start of htmlparser7a to get price data from digital look
class htmlParser7a(HTMLParser):
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.marketCapFound = False
		self.marketCap = ''
		self.sharesInIssueFound = False
		self.sharesInIssue = ''
		self.latestPrice = ''
		self.bidPrice = ''
		self.offerPrice = ''
		self.latestPriceFound = False
		self.bidPriceFound = False
		self.offerPriceFound = False


	def handle_starttag(self, tag, attrs):
		#print(tag, attrs)
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
		

	def handle_data(self, data):
		#print(data)
		if self.latestPriceFound:
			data = data.strip()
			if data:
				self.latestPrice = data
				self.latestPriceFound = False
		if data == 'Share Price':
			self.latestPriceFound = True
			
		if self.bidPriceFound:
			data = data.strip()
			if data:
				self.bidPrice = data
				self.bidPriceFound = False
		if data == 'Bid':
			self.bidPriceFound = True
			
		if self.offerPriceFound:
			data = data.strip()
			if data:
				self.offerPrice = data
				self.offerPriceFound = False
		if data == 'Offer':
			self.offerPriceFound = True
			
		if self.marketCapFound:
			data = data.strip()
			if data:
				self.marketCap = data
				self.marketCapFound = False
		if data.startswith('Market Cap:'):
			self.marketCapFound = True
			
		if self.sharesInIssueFound:
			data = data.strip()
			if data:
				self.sharesInIssue = data
				self.sharesInIssueFound = False
		if data.startswith('Shares Issued:'):
			self.sharesInIssueFound = True


			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			self.divList = []
			
		if tag == 'td':
			self.in_td = False

				
### end of htmlParser7a
#######################################################

#######################################################
### start of htmlparser7b to get dividends from digital look
class htmlParser7b(HTMLParser):
	global yrp2
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.pe = ''
		self.dividendYield = ''
		self.dividendCover = ''
		self.peFound = False
		self.valuationFound = False
		self.dividendYieldFound = False
		self.dividendCoverFound = False
		self.fundamentalsFound = False
		self.forecastsFound = False
		self.mainCurrency = ''
		self.divYr0 = ''
		self.divYrm1 = ''
		self.divYrm2 = ''
		self.divYrm3 = ''
		self.divYrm4 = ''
		self.divFcstYr0 = ''
		self.divFcstYrp1 = ''
		self.divFcstYrp2 = ''
		self.divList = []
		

	def handle_starttag(self, tag, attrs):
		#print(tag, attrs)
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
		

	def handle_data(self, data):

		if self.valuationFound and self.peFound:
			data = data.strip()
			if data:
				self.pe = data
				self.peFound = False
			
		if self.valuationFound and self.dividendYieldFound:
			data = data.strip()
			if data:
				self.dividendYield = data
				self.dividendYieldFound = False
				
		if self.valuationFound and self.dividendCoverFound:
			data = data.strip()
			if data:
				self.dividendCover = data
				self.dividendCoverFound = False
			
		if self.fundamentalsFound:
			data = data.strip()
			if data:
				self.divList.append(data)
				if len(self.divList) == 9:
					if self.divList[0].endswith(str(yr0)[2:]):
						self.divYr0 = self.divList[7]
					elif self.divList[0].endswith(str(yrm1)[2:]):
						self.divYrm1 = self.divList[7]
					elif self.divList[0].endswith(str(yrm2)[2:]):
						self.divYrm2 = self.divList[7]
					elif self.divList[0].endswith(str(yrm3)[2:]):
						self.divYrm3 = self.divList[7]
					elif self.divList[0].endswith(str(yrm4)[2:]):
						self.divYrm4 = self.divList[7]
						
		if self.forecastsFound:
			data = data.strip()
			if data:
				self.divList.append(data)
				if len(self.divList) == 9:
					if self.divList[0].endswith(str(yr0)[2:]):
						self.divFcstYr0 = self.divList[7]
					elif self.divList[0].endswith(str(yrp1)[2:]):
						self.divFcstYrp1 = self.divList[7]
					elif self.divList[0].endswith(str(yrp2)[2:]):
						self.divFcstYrp2 = self.divList[7]
		
		if self.fundamentalsFound:
			if data.startswith('Revenue'):
				self.mainCurrency = data[-3:-2]
				
		if data == 'P/E':
			self.peFound = True
		if data == 'Div Cover':
			self.dividendCoverFound = True
		if data == 'Div Yield':
			self.dividendYieldFound = True
		if data == 'Valuation':
			self.valuationFound = True
		if data == 'DPS':
			self.valuationFound = False
		if data.endswith('Fundamentals'):
			self.fundamentalsFound = True
		if data.endswith('Forecasts'):
			self.forecastsFound = True

			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			self.divList = []
			
			
		if tag == 'td':
			self.in_td = False
			
		if tag == 'table':
			self.fundamentalsFound = False
			self.forecastsFound = False

				
### end of htmlparser7b
#######################################################


#######################################################
### start of htmlparser7c to get dividends from digital look
class htmlParser7c(HTMLParser):
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.divYr0 = ''
		self.divYrm1 = ''
		self.divYrm2 = ''
		self.divYrm3 = ''
		self.divYrm4 = ''
		self.divList = [[]]
		self.recordDateFound = False
		self.rowCount = 0
		
	def handle_starttag(self, tag, attrs):
		#print(tag, attrs)
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
		

	def handle_data(self, data):
		data = data.strip()
		
		if self.recordDateFound and data:
			self.divList[self.rowCount].append(data)
			if len(self.divList[self.rowCount]) == 6:
				self.rowCount += 1
				self.divList.append([])
				
		if data == "Record Date":
			self.recordDateFound = True

			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			
		if tag == 'td':
			self.in_td = False
			
		if tag == 'table':
			self.recordDateFound = False
			
				
### end of htmlparser7c to get dividends from digital look
#######################################################

#######################################################
### htmlparser8 to get prices from iii
class htmlParser8(HTMLParser):
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.bidPrice = ''
		self.offerPrice = ''
		self.latestPrice = ''
		self.bidPriceFound = False
		self.offerPriceFound = False
		self.latestPriceFound = False
		
	def handle_starttag(self, tag, attrs):
		#print(tag, attrs)
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
		

	def handle_data(self, data):
		data = data.strip()
		
		if self.bidPriceFound and data:
			self.bidPrice = data
			self.bidPriceFound = False
		if data == 'Bid':
			self.bidPriceFound = True
			
		if self.offerPriceFound and data:
			self.offerPrice = data
			self.offerPriceFound = False
		if data == 'Ask':
			self.offerPriceFound = True
			
		if self.latestPriceFound and data:
			self.latestPrice = data
			self.latestPriceFound = False
		if data == 'Mid price':
			self.latestPriceFound = True

	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False

		if tag == 'td':
			self.in_td = False


### end of htmlparser8 to get prices from iii
#######################################################


#######################################################
### htmlparser8a to get data from iii
class htmlParser8a(HTMLParser):
	global yrp2
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	
	
	def __init__(self):
		HTMLParser.__init__(self)
		self.in_tr = False
		self.in_td = False
		self.marketCapFound = False
		self.marketCap = ''
		self.sharesInIssueFound = False
		self.sharesInIssue = ''
		self.pe = ''
		self.dividendYield = ''
		self.dividendCover = ''
		self.peFound = False
		self.dividendYieldFound = False
		self.dividendCoverFound = False
		self.dividendsFound = False
		self.divYrp1 = 0
		self.divYr0 = 0
		self.divYrm1 = 0
		self.divYrm2 = 0
		self.divYrm3 = 0
		self.divYrm4 = 0
		self.divCurYrp1 = ''
		self.divCurYr0 = ''
		self.divCurYrm1 = ''
		self.divCurYrm2 = ''
		self.divCurYrm3 = ''
		self.divCurYrm4 = ''
		self.divSpecYrp1 = False
		self.divSpecYr0 = False
		self.divSpecYrm1 = False
		self.divSpecYrm2 = False
		self.divSpecYrm3 = False
		self.divSpecYrm4 = False
		self.divList = []
		

	def handle_starttag(self, tag, attrs):
		#print(tag, attrs)
		if tag == 'tr' :
			self.in_tr = True
			
		if tag == 'td' :
			self.in_td = True
		

	def handle_data(self, data):
		data = data.strip()
		if data and self.marketCapFound:
			self.marketCap = data
			self.marketCapFound = False
		if data == 'Market Cap (GBP m)':
			self.marketCapFound = True
			
		if data and self.sharesInIssueFound:
			self.sharesInIssue = data
			self.sharesInIssueFound = False
		if data == 'Shares in issue (m)':
			self.sharesInIssueFound = True
			
		if data and self.peFound:
			self.pe = data
			self.peFound = False
		if data == 'PE Ratio':
			self.peFound = True
			
		if data and self.dividendYieldFound:
			self.dividendYield = data
			self.dividendYieldFound = False
		if data == 'Dividend Yield':
			self.dividendYieldFound = True
			
		
		if self.dividendsFound:
			data = data.strip()
			if data:
				self.divList.append(data)
				
				if len(self.divList) == 7:
					if self.divList[3].endswith(str(yrp1)):
						self.divYrp1 = self.divYrp1 + float(self.divList[4])
						self.divCurYrp1 = self.divList[5]
						if 'Special' in self.divList[6]: self.divSpecYrp1 = True
					if self.divList[3].endswith(str(yr0)):
						self.divYr0 = self.divYr0 + float(self.divList[4])
						self.divCurYr0 = self.divList[5]
						if 'Special' in self.divList[6]: self.divSpecYr0 = True
					elif self.divList[3].endswith(str(yrm1)):
						self.divYrm1 = self.divYrm1 + float(self.divList[4])
						self.divCurYrm1 = self.divList[5]
						if 'Special' in self.divList[6]: self.divSpecYm1 = True
					elif self.divList[3].endswith(str(yrm2)):
						self.divYrm2 = self.divYrm2 + float(self.divList[4])
						self.divCurYrm2 = self.divList[5]
						if 'Special' in self.divList[6]: self.divSpecYrm2 = True
					elif self.divList[3].endswith(str(yrm3)):
						self.divYrm3 = self.divYrm3 + float(self.divList[4])
						self.divCurYrm3 = self.divList[5]
						if 'Special' in self.divList[6]: self.divSpecYrm3 = True
					elif self.divList[3].endswith(str(yrm4)):
						self.divYrm4 = self.divYrm4 + float(self.divList[4])
						self.divCurYrm4 = self.divList[5]
						if 'Special' in self.divList[6]: self.divSpecYrm4 = True
		if data == 'Dividends':
			self.dividendsFound = True

			
	def handle_endtag(self, tag):
		
		if tag == 'tr':
			self.in_tr = False
			self.divList = []
			
		if tag == 'td':
			self.in_td = False
			self.dividendYieldFound = False
			self.marketCapFound = False

		if tag == 'table':
			self.dividendsFound = False

### end of htmlparser8a to get data from iii
#######################################################

#######################################################
### get year data
def functionGetYears():
	global yrp2
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	
	now = datetime.datetime.now()
	yr0 = now.year
	yrp2 = yr0 + 2
	yrp1 = yr0 + 1
	yrm1 = yr0 - 1
	yrm2 = yr0 - 2
	yrm3 = yr0 - 3
	yrm4 = yr0 - 4

	return
### end of functionGetYears
#######################################################

#######################################################
### get yahooSymbol, MorningstarId, DL CSI, iii symbol
def functionEpic():
	global parentwin
	global epic
	global mainSheet
	global epicSheet
	global yahooSymbol
	global iiiSymbol
	global dlCSI
	global mStarId
	
	epicFound = False
	epic = mainSheet.getCellByPosition(2,0).String
	epic = epic.upper()
	
	for rowNum in range(1,500):
		if epicSheet.getCellByPosition(0,rowNum).String == epic:
			epicFound = True
			break
	
	if epicFound:
		mStarId = epicSheet.getCellByPosition(4,rowNum).String
		yahooSymbol = epicSheet.getCellByPosition(5,rowNum).String
		dlCSI = epicSheet.getCellByPosition(6,rowNum).String
		iiiSymbol = epicSheet.getCellByPosition(7,rowNum).String
	else:
		messageBox("Epic not found in EpicSheet", "Error")
		return("error")
	
	
	return
### end of functionEpic
#######################################################

#######################################################
### set hyperlinks in column A
def functionSetHyperlink(label, url, col,row):

	global mainSheet
	
	oCell = mainSheet.getCellByPosition(col,row)
	oText = oCell.getText()
	
	oLink = oDoc.createInstance("com.sun.star.text.TextField.URL")
	oLink.URL = url
	oLink.Representation = label
	
	oCell.insertTextContent(oText.createTextCursor(),oLink,True)
	
	return

### end of set hyperlinks
#######################################################
	
#######################################################
### get data from ADVFN
def function_1():
	global parentwin
	global epic
	global wwwAddress
	global marketCap
	global sharesInIssue
	global pe
	global dividendYield
	global dividendCover
	global grossGearing
	global latestPrice
	global bidPrice
	global offerPrice
	global marketCapRow
	global sharesInIssueRow
	global peRow
	global dividendYieldRow
	global dividendCoverRow
	global grossGearingRow
	global url
	global divYrp1
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global divCurYrp1
	global divCurYr0
	global divCurYrm1
	global divCurYrm2
	global divCurYrm3
	global divCurYrm4
	global divSpecYrp1
	global divSpecYr0
	global divSpecYrm1
	global divSpecYrm2
	global divSpecYrm3
	global divSpecYrm4
	global webTimeOut
	global sector
	global fund
	global webTimeOut

	urlBase = 'http://uk.advfn.com/p.php?pid=financials&btn=s_ok&symbol=LSE%3A'
	url = urlBase + epic
	
	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing ADVFN','Error')
		return
	
	try:
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser1()
		p.feed(html)
		
		wwwAddress = p.wwwAddress
		sector = p.sector

		if sector.startswith('EQUITY INVESTMENT INSTR'):
			fund = True
		marketCap = p.marketCap.replace(',','')
		if marketCap.endswith('m'):
			marketCap = marketCap.replace('m','')
		sharesInIssue = p.sharesInIssue.replace(',','')
		if sharesInIssue.endswith('m'):
			sharesInIssue = sharesInIssue.replace('m','')
		pe = p.pe.replace(',','')
		pe = pe.replace('-','')
		dividendYield = p.dividendYield.replace(',','')
		dividendYield = dividendYield.replace('%','')
		dividendYield = dividendYield.replace('-','')
		dividendCover = p.dividendCover.replace(',','')
		dividendCover = dividendCover.replace('-','')
		grossGearing = p.grossGearing.replace(',','')
		divYrp1 = round(p.divYrp1,5)
		divYr0 = round(p.divYr0,5)
		divYrm1 = round(p.divYrm1,5)
		divYrm2 = round(p.divYrm2,5)
		divYrm3 = round(p.divYrm3,5)
		divYrm4 = round(p.divYrm4,5)
		divCurYrp1 = p.divCurYrp1
		divCurYr0 = p.divCurYr0
		divCurYrm1 = p.divCurYrm1
		divCurYrm2 = p.divCurYrm2
		divCurYrm3 = p.divCurYrm3
		divCurYrm4 = p.divCurYrm4
		divSpecYrp1 = p.divSpecYrp1
		divSpecYr0 = p.divSpecYr0
		divSpecYrm1 = p.divSpecYrm1
		divSpecYrm2 = p.divSpecYrm2
		divSpecYrm3 = p.divSpecYrm3
		divSpecYrm4 = p.divSpecYrm4
		latestPrice = p.priceList[0].replace(',','')
		latestPrice = float(latestPrice)
		bidPrice = p.priceList[2].replace(',','')
		bidPrice = float(bidPrice)
		offerPrice = p.priceList[3].replace(',','')
		offerPrice = float(offerPrice)
	except:
		messageBox("Problems parsing HTML data from ADVFN\nClick OK to continue","Error")
	
	return

### end of get data from ADVFN
#######################################################

######################################################
### get data from MoneyAM
def function_2():
	global epic
	global marketCap
	global sharesInIssue
	global pe
	global dividendYield
	global dividendCover
	global grossGearing
	global url
	global divYrp1
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global divCurYrp1
	global divCurYr0
	global divCurYrm1
	global divCurYrm2
	global divCurYrm3
	global divCurYrm4
	global divSpecYrp1
	global divSpecYr0
	global divSpecYrm1
	global divSpecYrm2
	global divSpecYrm3
	global divSpecYrm4
	global latestPrice
	global bidPrice
	global offerPrice
	global parentwin
	global webTimeOut

	urlBase = 'http://www.moneyam.com/fundamentals/' 
	url = urlBase + epic

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing MoneyAM','Error')
		return
	
	try:
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser2()
		p.feed(html)

		marketCap = p.marketCap.replace(',','')
		sharesInIssue = p.sharesInIssue.replace(',','')
		pe = p.pe.replace(',','')
		
		dividendYield = p.dividendYield.replace(',','')
		dividendYield = dividendYield.replace('%','')
		dividendCover = p.dividendCover.replace(',','')
		grossGearing = p.grossGearing.replace(',','')
		grossGearing = grossGearing.replace('n/a','')
		divYrp1 = round(p.divYrp1,5)
		divYr0 = round(p.divYr0,5)
		divYrm1 = round(p.divYrm1,5)
		divYrm2 = round(p.divYrm2,5)
		divYrm3 = round(p.divYrm3,5)
		divYrm4 = round(p.divYrm4,5)
		divCurYrp1 = p.divCurYrp1
		divCurYr0 = p.divCurYr0
		divCurYrm1 = p.divCurYrm1
		divCurYrm2 = p.divCurYrm2
		divCurYrm3 = p.divCurYrm3
		divCurYrm4 = p.divCurYrm4
		divSpecYrp1 = p.divSpecYrp1
		divSpecYr0 = p.divSpecYr0
		divSpecYrm1 = p.divSpecYrm1
		divSpecYrm2 = p.divSpecYrm2
		divSpecYrm3 = p.divSpecYrm3
		divSpecYrm4 = p.divSpecYrm4
		latestPrice = p.latestPrice.replace(',','')
		latestPrice = float(latestPrice)
		bidPrice = p.bidPrice.replace(',','')
		bidPrice = float(bidPrice)
		offerPrice = p.offerPrice.replace(',','')
		offerPrice = float(offerPrice)
	except:
		messageBox("Problems parsing HTML data from MoneyAM\nClick OK to continue","Error")

	return

## end of get data from MoneyAM
######################################################


######################################################
## get data from Yahoo
def function_3():
	global marketCap
	global sharesInIssue
	global pe
	global dividendYield
	global dividendCover
	global grossGearing
	global yahooSymbol
	global url
	global latestPrice
	global bidPrice
	global offerPrice
	global webTimeOut

	urlBase = 'http://finance.yahoo.com/q?s='
	url = urlBase + yahooSymbol

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Yahoo','Error')
		return
	
	try:	
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser3()
		p.feed(html)
		
		marketCap = p.marketCap.replace(',','')
		marketCap = p.marketCap.replace('N/A','')
		if marketCap.endswith('B'):
			marketCap = marketCap.replace('B','')
			marketCap = float(marketCap) * 1000
		
		latestPrice = p.latestPrice.replace(',','')
		bidPrice = p.bidPrice.replace(',','')
		offerPrice = p.offerPrice.replace(',','')
	except:
		messageBox("Problems parsing HTML data from Yahoo\nClick OK to continue","Error")

	return

## end of get data from Yahoo
######################################################

######################################################
## get data from Morningstar
def function_4():
	global marketCap
	global sharesInIssue
	global pe
	global dividendYield
	global dividendCover
	global grossGearing
	global mStarId
	global url
	global divFcstYrp2
	global divFcstYrp1
	global divFcstYr0
	global parentwin
	global latestPrice
	global bidPrice
	global offerPrice
	global webTimeOut

	# mStarId = '0P000090TU'
	urlBase = 'http://tools.morningstar.co.uk/uk/stockreport/default.aspx?Site=uk&id=mstarid&LanguageId=en-GB&SecurityToken=mstarid]3]0]E0WWE$$ALL'
	url = urlBase
	url = url.replace('mstarid',mStarId)
	
	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Morningstar','Error')
		return
		
	try:
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser4()
		p.feed(html)
		
		marketCap = p.marketCap.replace(',','')
		if marketCap.endswith('B'):
			marketCap = marketCap.replace('B','')
			marketCap = float(marketCap) * 1000
		sharesInIssue = p.sharesInIssue.replace(',','')
		pe = p.pe.replace(',','')
		dividendYield = p.dividendYield.replace(',','')
		dividendCover = p.dividendCover.replace(',','')
		grossGearing = p.grossGearing.replace(',','')
		divFcstYrp2 = float(p.divFcstYrp2)
		divFcstYrp1 = float(p.divFcstYrp1)
		divFcstYr0 = float(p.divFcstYr0)
		latestPrice = p.latestPrice.replace(',','')
		bidPrice = p.bidPrice.replace(',','')
		offerPrice = p.offerPrice.replace(',','')
	except:
		msg = "Problems parsing HTML data from Morningstar\n"
		msg = msg + "Click OK to continue"
		messageBox(msg,"Error")
	return

## end of get data from Morningstar
######################################################

######################################################
## get dividends from Morningstar
def function_4a():

	global mStarId
	global url
	global divFcstYr0
	global parentwin
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global webTimeOut
	global parentwin

	# mStarId = '0P000090TU'
	urlBase = 'http://tools.morningstar.co.uk/uk/stockreport/default.aspx?tab=10&vw=fs&SecurityToken=mstarid]3]0]E0WWE$$ALL&Id=mstarid&ClientFund=0&CurrencyId=GBP'
	url = urlBase
	url = url.replace('mstarid',mStarId)

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Morningstar','Error')
		return
		
	try:
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser4a()
		p.feed(html)
		
		if p.y1 == str(yr0):
			divYr0 = p.dpsY1
		elif p.y1 == str(yrm1):
			divYrm1 = p.dpsY1
		elif p.y1 == str(yrm2):
			divYrm2 = p.dpsY1
		elif p.y1 == str(yrm3):
			divYrm3 = p.dpsY1
		elif p.y1 == str(yrm4):
			divYrm4 = p.dpsY1
			
		if p.y2 == str(yr0):
			divYr0 = p.dpsY2
		elif p.y2 == str(yrm1):
			divYrm1 = p.dpsY2
		elif p.y2 == str(yrm2):
			divYrm2 = p.dpsY2
		elif p.y2 == str(yrm3):
			divYrm3 = p.dpsY2
		elif p.y2 == str(yrm4):
			divYrm4 = p.dpsY2
			
		if p.y3 == str(yr0):
			divYr0 = p.dpsY3
		elif p.y3 == str(yrm1):
			divYrm1 = p.dpsY3
		elif p.y3 == str(yrm2):
			divYrm2 = p.dpsY3
		elif p.y3 == str(yrm3):
			divYrm3 = p.dpsY3
		elif p.y3 == str(yrm4):
			divYrm4 = p.dpsY3
		
		if p.y4 == str(yr0):
			divYr0 = p.dpsY4
		elif p.y4 == str(yrm1):
			divYrm1 = p.dpsY4
		elif p.y4 == str(yrm2):
			divYrm2 = p.dpsY4
		elif p.y4 == str(yrm3):
			divYrm3 = p.dpsY4
		elif p.y4 == str(yrm4):
			divYrm4 = p.dpsY4
			
		if p.y5 == str(yr0):
			divYr0 = p.dpsY5
		elif p.y5 == str(yrm1):
			divYrm1 = p.dpsY5
		elif p.y5 == str(yrm2):
			divYrm2 = p.dpsY5
		elif p.y5 == str(yrm3):
			divYrm3 = p.dpsY5
		elif p.y5 == str(yrm4):
			divYrm4 = p.dpsY5
	except:
		messageBox("Problems parsing HTML data from Morningstar\nClick OK to continue","Error")
		
	return

## end of get function_4a to get dividends from Morningstar
######################################################

######################################################
## get data from MSN Money
def function_5():
	global bidPrice
	global offerPrice
	global latestPrice
	global marketCap
	global sharesInIssue
	global pe
	global dividendYield
	global dividendCover
	global grossGearing
	global epic
	global url
	global webTimeOut

	urlBase = 'http://uk.moneycentral.msn.com/investor/quotes/quotes.aspx?Symbol=$GB:'
	url = urlBase + epic

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing MSN Money','Error')
		return
		
	try:
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser5()
		p.feed(html)
		
		bidPrice = p.bidPrice.replace(',','')
		offerPrice = p.offerPrice.replace(',','')
		latestPrice = p.latestPrice.replace(',','')

		marketCap = p.marketCap.replace(',','')
		if marketCap.endswith(' Bil'):
			marketCap = marketCap.replace(' Bil','')
			marketCap = float(marketCap) * 1000
		sharesInIssue = p.sharesInIssue.replace(',','')
		if sharesInIssue.endswith(' Mil'):
			sharesInIssue = sharesInIssue.replace(' Mil','')
		elif sharesInIssue.endswith(' Bil'):
			sharesInIssue = sharesInIssue.replace(' Bil','')
			sharesInIssue = sharesInIssue.replace('.','')
			sharesInIssue = sharesInIssue + '000'
		pe = p.pe.replace(',','')
		pe = pe.replace('NA','')
		dividendYield = p.dividendYield.replace(',','')
		dividendYield = dividendYield.replace('NA','')
		dividendCover = p.dividendCover.replace(',','')
		grossGearing = p.grossGearing.replace(',','')
	except:
		messageBox("Problems parsing HTML data from MSN Money\nClick OK to continue","Error")

	return

## end of function_5 to get data from MSN Money
######################################################

######################################################
## function_6 to get dividends from dividend investor
def function_6():

	global epic
	global url
	global divYrp1
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global divSpecYrp1
	global divSpecYr0
	global divSpecYrm1
	global divSpecYrm2
	global divSpecYrm3
	global divSpecYrm4
	global webTimeOut


	urlBase = 'http://www.dividendinvestor.co.uk/historical.php?exg=GB&symbol='
	url = urlBase
	url = url + epic

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Dividend Investor','Error')
		return
		
	try:
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser6()
		p.feed(html)
		
		divYrp1 = round(p.divYrp1,5)
		divYr0 = round(p.divYr0,5)
		divYrm1 = round(p.divYrm1,5)
		divYrm2 = round(p.divYrm2,5)
		divYrm3 = round(p.divYrm3,5)
		divYrm4 = round(p.divYrm4,5)
		divSpecYrp1 = p.divSpecYrp1
		divSpecYr0 = p.divSpecYr0
		divSpecYrm1 = p.divSpecYrm1
		divSpecYrm2 = p.divSpecYrm2
		divSpecYrm3 = p.divSpecYrm3
		divSpecYrm4 = p.divSpecYrm4
	except:
		messageBox("Problems parsing HTML data from Dividend Investor\nClick OK to continue","Error")


	return

## end of function_6 to get dividends from dividend investor
######################################################

######################################################
## get currency from dividend investor
def function_6a():

	global epic
	global url
	global divCur
	global marketCap
	global sharesInIssue
	global dividendYield
	global latestPrice
	global webTimeOut

	urlBase = 'http://www.dividendinvestor.co.uk/?chk=868ee1387197349&symbol='
	url = urlBase
	url = url + epic

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Dividend Investor','Error')
		return
	
	try:
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser6a()
		p.feed(html)

		marketCap = p.marketCap
		marketCap = marketCap.replace(',','')
		marketCap = marketCap.replace('M','')
		sharesInIssue = p.sharesInIssue
		sharesInIssue = sharesInIssue.replace(',','')
		sharesInIssue = sharesInIssue.replace('M','')
		dividendYield = p.dividendYield.replace(' %','')
		divCur = p.divCur
		latestPrice = p.latestPrice
		latestPrice = latestPrice.replace(' ','')
		latestPrice = latestPrice.replace(',','')
	except:
		messageBox("Problems parsing HTML data from Dividend Investor\nClick OK to continue","Error")

	return


## end of function_6a
######################################################

######################################################
## get price data from digital look
def function_7a():
	global dlCSI
	global marketCap
	global sharesInIssue
	global latestPrice
	global bidPrice
	global offerPrice
	global url
	global parentwin
	global webTimeOut

	urlBase = 'http://www.digitallook.com/companyresearch/DLCSI/Admiral_Group/share_prices.html' 
	url = urlBase.replace('DLCSI',dlCSI)
	
	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Digital Look 7a','Error')
		return
		
	try:
		html = response.read()
		html = html.decode('1252','ignore')
		
		p = htmlParser7a()
		p.feed(html)
		
		latestPrice = p.latestPrice.replace(',','')
		bidPrice = p.bidPrice.replace(',','')
		offerPrice = p.offerPrice.replace(',','')
		
		marketCap = p.marketCap.replace(',','')
		
		matchObj = re.search( '.(\d*).', marketCap)
		if matchObj:
			marketCap = matchObj.group(1)
		
		sharesInIssue = p.sharesInIssue.replace(',','')
		sharesInIssue = sharesInIssue.replace('m','')
	except:
		messageBox("Problems parsing HTML data from Digital Look 7a\nClick OK to continue","Error")

	return

## end of function_7a
######################################################

######################################################
## start of function_7b to get dividends from digital look
def function_7b():
	global dlCSI
	global marketCap
	global sharesInIssue

	global latestPrice
	global bidPrice
	global offerPrice
	global pe
	global dividendYield
	global dividendCover
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global divCurYr0
	global divCurYrm1
	global divCurYrm2
	global divCurYrm3
	global divCurYrm4
	global divFcstYr0
	global divFcstYrp1
	global divFcstYrp2
	global mainCurrency
	global url
	global parentwin
	global webTimeOut

	urlBase = 'http://www.digitallook.com/cgi-bin/dlmedia/security.cgi?username=&ac=&csi=' 
	url = urlBase + dlCSI

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Digital Look 7b','Error')
		return
	
	try:
		html = response.read()
		html = html.decode('1252','ignore')
		html = html.replace('&cent;','c')
			
		p = htmlParser7b()
		p.feed(html)
		
		pe = p.pe.replace('n/a','')
		dividendYield = p.dividendYield.replace('0.0%','')
		dividendCover = p.dividendCover
		dividendCover = dividendCover.replace('n/a','')
		mainCurrency = p.mainCurrency
		divYr0 = p.divYr0
		divYrm1 = p.divYrm1
		divYrm2 = p.divYrm2
		divYrm3 = p.divYrm3
		divYrm4 = p.divYrm4
		divFcstYr0 = p.divFcstYr0
		divFcstYrp1 = p.divFcstYrp1
		divFcstYrp2 = p.divFcstYrp2
	except:
		messageBox("Problems parsing HTML data from Digital Look 7b\nClick OK to continue","Error")
	
	return

## end of function_7b
######################################################



######################################################
## start of function_7c to get dividends from digital look
def function_7c():
	global dlCSI
	global divYrp1
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global divCurYrp1
	global divCurYr0
	global divCurYrm1
	global divCurYrm2
	global divCurYrm3
	global divCurYrm4
	global divSpecYrp1 
	global divSpecYr0 
	global divSpecYrm1
	global divSpecYrm2
	global divSpecYrm3
	global divSpecYrm4
	global url
	global parentwin
	global webTimeOut

	urlBase = 'http://www.digitallook.com/cgi-bin/dlmedia/security.cgi?csi=DLCSI&action=financials&sub_action=dividends' 
	url = urlBase.replace('DLCSI',dlCSI)

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Digital Look 7c','Error')
		return
	
	try:
		html = response.read()
		html = html.decode('1252','ignore')
		html = html.replace('&nbsp;','xxxx')	# fill space in dividends table with xxxx
		html = html.replace('&cent;','c')		# replace cents in dividends table with c
			
		p = htmlParser7c()
		p.feed(html)
		
		for x in range(0,len(p.divList)-1):
			if p.divList[x][3][-2:] == str(yrp1)[-2:]:
				divYrp1 = divYrp1 + float(p.divList[x][2][:-1])
				divCurYrp1 = p.divList[x][2][-1:]
				if p.divList[x][1] == 'Special': divSpecYrp1 = True
			elif p.divList[x][3][-2:] == str(yr0)[-2:]:
				divYr0 = divYr0 + float(p.divList[x][2][:-1])
				divCurYr0 = p.divList[x][2][-1:]
				if p.divList[x][1] == 'Special': divSpecYr0 = True
			elif p.divList[x][3][-2:] == str(yrm1)[-2:]:
				divYrm1 = divYrm1 + float(p.divList[x][2][:-1])
				divCurYrm1 = p.divList[x][2][-1:]
				if p.divList[x][1] == 'Special': divSpecYrm1 = True
			elif p.divList[x][3][-2:] == str(yrm2)[-2:]:
				divYrm2 = divYrm2 + float(p.divList[x][2][:-1])
				divCurYrm2 = p.divList[x][2][-1:]
				if p.divList[x][1] == 'Special': divSpecYrm2 = True
			elif p.divList[x][3][-2:] == str(yrm3)[-2:]:
				divYrm3 = divYrm3 + float(p.divList[x][2][:-1])
				divCurYrm3 = p.divList[x][2][-1:]
				if p.divList[x][1] == 'Special': divSpecYrm3 = True
			elif p.divList[x][3][-2:] == str(yrm4)[-2:]:
				divYrm4 = divYrm4 + float(p.divList[x][2][:-1])
				divCurYrm4 = p.divList[x][2][-1:]
				if p.divList[x][1] == 'Special': divSpecYrm4 = True
		
	except:
		messageBox("Problems parsing HTML data from Digital Look 7c\nClick OK to continue","Error")
	
	
	return

## end of function_7c
######################################################


######################################################
## function_8 to get prices from iii
def function_8():
	global iiiSymbol
	global latestPrice
	global bidPrice
	global offerPrice
	global url
	global parentwin
	global webTimeOut

	urlBase = 'http://www.iii.co.uk/investment/detail?code=cotn:symbol&display=trades'  
	url = urlBase.replace('symbol', iiiSymbol)

	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Interactive Investor','Error')
		return
		
	try:
		html = response.read()
		html = html.decode('1252','ignore')
			
		p = htmlParser8()
		p.feed(html)
		
		bidPrice = p.bidPrice
		offerPrice = p.offerPrice
		latestPrice = p.latestPrice
	except:
		messageBox("Problems parsing HTML data from Interactive Investor\nClick OK to continue","Error")

	return

## end of function_8 to get prices from iii
######################################################

######################################################
## function_8a to get data from iii
def function_8a():
	global epic
	global marketCap
	global sharesInIssue

	global latestPrice
	global bidPrice
	global offerPrice
	global pe
	global dividendYield
	global dividendCover
	global divYrp1
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global divCurYrp1
	global divCurYr0
	global divCurYrm1
	global divCurYrm2
	global divCurYrm3
	global divCurYrm4
	global divSpecYrp1
	global divSpecYr0
	global divSpecYrm1
	global divSpecYrm2
	global divSpecYrm3
	global divSpecYrm4
	global url
	global parentwin
	global webTimeOut

	urlBase = 'http://www.iii.co.uk/research/LSE:symbol/fundamentals'
	url = urlBase.replace('symbol', epic)
	
	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		messageBox('Problem accessing Interactive Investor','Error')
		return
		
	try:
		html = response.read()
		html = html.decode('1252','ignore')
		html = html.replace('&nbsp;','')
			
		p = htmlParser8a()
		p.feed(html)
		
		marketCap = p.marketCap
		marketCap = marketCap.replace(',','')
		sharesInIssue = p.sharesInIssue
		sharesInIssue = sharesInIssue.replace(',','')
		pe = p.pe
		if pe.startswith('n/a'): pe = ''
		dividendYield = p.dividendYield
		dividendCover = p.dividendCover
		
		matchObj = re.search( '\d', dividendCover)
		if matchObj:
			dividendCover = dividendCover
		else:
			dividendCover = ''
		
		divYrp1 = round(p.divYrp1,5)
		divCurYrp1 = p.divCurYrp1
		divSpecYrp1 = p.divSpecYrp1
		divYr0 = round(p.divYr0,5)
		divCurYr0 = p.divCurYr0
		divSpecYr0 = p.divSpecYr0
		divYrm1 = round(p.divYrm1,5)
		divCurYrm1 = p.divCurYrm1
		divSpecYrm1 = p.divSpecYrm1
		divYrm2 = round(p.divYrm2,5)
		divCurYrm2 = p.divCurYrm2
		divSpecYrm2 = p.divSpecYrm2
		divYrm3 = round(p.divYrm3,5)
		divCurYrm3 = p.divCurYrm3
		divSpecYrm3 = p.divSpecYrm3
		divYrm4 = round(p.divYrm4,5)
		divCurYrm4 = p.divCurYrm4
		divSpecYrm4 = p.divSpecYrm4
	except:
		messageBox("Problems parsing HTML data from Interactive Investor\nClick OK to continue","Error")
	

	return

## end of function_8a to get data from iii
#######################################################


#######################################################
### set row labels
def functionRowLabel():
	global oDoc
	global mainSheet
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	global divYrp1Row
	global divYr0Row
	global divYrm1Row
	global divYrm2Row
	global divYrm3Row
	global divYrm4Row
	
	mainSheet.getCellByPosition(0,divFcstYrp2Row - 1).String = 'Fcst Div ' + str(yrp2)
	mainSheet.getCellByPosition(0,divFcstYrp1Row - 1).String = 'Fcst Div ' + str(yrp1)
	mainSheet.getCellByPosition(0,divFcstYr0Row - 1).String = 'Fcst Div ' + str(yr0)
	mainSheet.getCellByPosition(0,divYrp1Row - 1).String = 'Actual Div ' + str(yrp1)
	mainSheet.getCellByPosition(0,divYr0Row - 1).String = 'Actual Div ' + str(yr0)
	mainSheet.getCellByPosition(0,divYrm1Row - 1).String = 'Actual Div ' + str(yrm1)
	mainSheet.getCellByPosition(0,divYrm2Row - 1).String = 'Actual Div ' + str(yrm2)
	mainSheet.getCellByPosition(0,divYrm3Row - 1).String = 'Actual Div ' + str(yrm3)
	mainSheet.getCellByPosition(0,divYrm4Row - 1).String = 'Actual Div ' + str(yrm4)
	
	return

### end of set labels
#######################################################

#######################################################
### Function to check current version of spreadsheet
def functionVersion():
	global webTimeOut
	
	url = 'http://tmffinancialsoftware.weebly.com/sharedatadump.html' 
	
	try:
		if sys.version[0:1] == '3':
			req = urllib.request.Request(url=url, headers={'User-Agent' : "Magic Browser"})
			response = urllib.request.urlopen(req, timeout = webTimeOut)
		else:
			req = urllib2.Request(url=url)
			req.add_header('User-agent', 'Magic Browser')
			response = urllib2.urlopen(req, timeout = webTimeOut)
	except:
		return
	
	try:
		html = response.read()
		html = html.decode('1252','ignore')
		html = html.replace('&nbsp;','')
	
		if 'v1_10' not in html:
			msg = "There is a newer version of the spreadsheet on the Weebly web page.\n"
			msg = msg + "Click on the Help button to go to Weebly, then you can download.\n\n"
			msg = msg + "If you want to continue using this version without getting this warning message,\n"
			msg = msg + "go to the Parameters sheet and disable the check for updates\n\n"
			messageBox(msg,"New version available")
			return 1
	except:
		return 0
	
	
	return
### end of Function to check current version of spreadsheet
#######################################################

#######################################################
### Resets variables to default values
def resetValues():
	global marketCap
	global sharesInIssue
	global pe
	global dividendYield
	global dividendCover
	global grossGearing
	global latestPrice
	global bidPrice
	global offerPrice
	global mainCurrency
	global divFcstYrp2
	global divFcstYrp1
	global divFcstYr0
	global divYrp1
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global divCur
	global divCurYrp1
	global divCurYr0
	global divCurYrm1
	global divCurYrm2
	global divCurYrm3
	global divCurYrm4
	global divSpecYrp1
	global divSpecYr0
	global divSpecYrm1
	global divSpecYrm2
	global divSpecYrm3
	global divSpecYrm4
	
	
	marketCap = ''
	sharesInIssue = ''
	pe = ''
	dividendYield = ''
	dividendCover = ''
	grossGearing = ''
	mainCurrency = ''
	latestPrice = ''
	bidPrice = ''
	offerPrice = ''
	divFcstYrp2 = 0
	divFcstYrp1 = 0
	divFcstYr0 = 0
	divYrp1 = 0
	divYr0 = 0
	divYrm1 = 0
	divYrm2 = 0
	divYrm3 = 0
	divYrm4 = 0
	divCur = ''
	divCurYrp1 = ''
	divCurYr0 = ''
	divCurYrm1 = ''
	divCurYrm2 = ''
	divCurYrm3 = ''
	divCurYrm4 = ''
	divSpecYrp1 = False
	divSpecYr0 = False
	divSpecYrm1 = False
	divSpecYrm2 = False
	divSpecYrm3 = False
	divSpecYrm4 = False
	
	return

### end of resetValues
#######################################################

####################################################################################
####################################################################################
####################################################################################

### define variables
marketCap = ''
sharesInIssue = ''
wwwAddress = ''
pe = ''
dividendYield = ''
dividendCover = ''
grossGearing = ''
mainCurrency = ''
mainCurrencyTemp = ''
latestPrice = ''
bidPrice = ''
offerPrice = ''
parentwin = ''
mainSheet = ''
epicSheet = ''
epic = ''
yahooSymbol = ''
iiiSymbol = ''
dlCSI = ''
mStarId = ''
url = ''
yrp2 = 0
yrp1 = 0
yr0 = 0
yrm1 = 0
yrm2 = 0
yrm3 = 0
yrm4 = 0
divFcstYrp2 = 0
divFcstYrp1 = 0
divFcstYr0 = 0
divYrp1 = 0
divYr0 = 0
divYrm1 = 0
divYrm2 = 0
divYrm3 = 0
divYrm4 = 0
divCur = ''
divCurYrp1 = ''
divCurYr0 = ''
divCurYrm1 = ''
divCurYrm2 = ''
divCurYrm3 = ''
divCurYrm4 = ''
divSpecYrp1 = False
divSpecYr0 = False
divSpecYrm1 = False
divSpecYrm2 = False
divSpecYrm3 = False
divSpecYrm4 = False
oDoc = ''
mainSheet = ''
divFcstYrp2Row = 0
divFcstYrp1Row = 0
divFcstYr0Row = 0
divYrp1Row = 0
divYr0Row = 0
divYrm1Row = 0
divYrm2Row = 0
divYrm3Row = 0
divYrm4Row = 0
webTimeOut = 10
sector = ''
fund = False

	
	
	
def main_function():
	
	global marketCap
	global sharesInIssue
	global pe
	global wwwAddress
	global dividendYield
	global dividendCover
	global grossGearing
	global mainCurrency
	global mainCurrencyTemp
	global latestPrice
	global bidPrice
	global offerPrice
	global parentwin
	global mainSheet
	global epicSheet
	global epic
	global mStarId
	global dlCSI
	global iiiSymbol
	global url
	global divFcstYrp2
	global divFcstYrp1
	global divFcstYr0
	global divYrp1
	global divYr0
	global divYrm1
	global divYrm2
	global divYrm3
	global divYrm4
	global divCur
	global divCurYrp1
	global divCurYr0
	global divCurYrm1
	global divCurYrm2
	global divCurYrm3
	global divCurYrm4
	global divSpecYrp1 
	global divSpecYr0 
	global divSpecYrm1
	global divSpecYrm2
	global divSpecYrm3
	global divSpecYrm4
	global oDoc
	global mainSheet
	global divFcstYrp2Row
	global divFcstYrp1Row
	global divFcstYr0Row
	global divYrp1Row
	global divYr0Row
	global divYrm1Row
	global divYrm2Row
	global divYrm3Row
	global divYrm4Row
	global yrp2
	global yrp1
	global yr0
	global yrm1
	global yrm2
	global yrm3
	global yrm4
	global sector
	global fund
	global webTimeOut
	
	oDoc = XSCRIPTCONTEXT.getDocument()
	parentwin = oDoc.CurrentController.Frame.ContainerWindow
	
	oSheets = oDoc.getSheets()
	mainSheet = oSheets.getByName("MainSheet")
	epicSheet = oSheets.getByName("EpicSheet")
	
	paramSheet = oSheets.getByName("Parameters")
	webTimeOut = paramSheet.getCellRangeByName("B2").Value		# get value for webTimeOut
	if not webTimeOut:
		webTimeOut = 10
	
	# set up status bar
	Controller = oDoc.getCurrentController()
	DocFrame = Controller.getFrame()
	oStatusIndicator = DocFrame.createStatusIndicator()
	oStatusIndicator.start( "", 10 )
	
	### spreadsheet output rows
	latestPriceRow = 4
	bidPriceRow = 5
	offerPriceRow = 6
	marketCapRow = 8
	sharesInIssueRow = 9
	peRow = 10
	dividendYieldRow = 12
	dividendCoverRow = 13
	grossGearingRow = 14
	divFcstYrp2Row = 16
	divFcstYrp1Row = 17
	divFcstYr0Row = 18
	divYrp1Row = 20
	divYr0Row = 21
	divYrm1Row = 22
	divYrm2Row = 23
	divYrm3Row = 24
	divYrm4Row = 25
	
	# check current version
	checkVersion = paramSheet.getCellRangeByName("B3").String
	if checkVersion == 'Yes':
		version = functionVersion()
		if version == 1:
			return
	
	# clear sheet contents
	mainSheet.getCellRangeByName("B3:I30").clearContents(5)		# clear the main data block
	mainSheet.getCellRangeByName("C2").clearContents(5)		# clear the company website link
	
	# get current, future and past years
	functionGetYears()
	
	# Set row labels
	functionRowLabel()
	
	### Get Epic data
	retCode = functionEpic()
	if retCode == "error": return
	if not mStarId or not epic or not dlCSI or not yahooSymbol or not iiiSymbol:
		msg = "Incomplete data found in EpicSheet" + "\n"
		msg = msg + "Epic: " + epic + "\n"
		msg = msg + "Morningstar ID: " + mStarId + "\n"
		msg = msg + "Digital Look CSI: " + dlCSI + "\n"
		msg = msg + "Yahoo symbol: " + yahooSymbol + "\n"
		msg = msg + "iii symbol: " + iiiSymbol + "\n"
		messageBox(msg, "Missing Data")

	
	## get data from ADVFN
	oStatusIndicator.setText("Processing ADVFN")
	oStatusIndicator.setValue(1)
	function_1()
	
	# write to spreadsheet
	functionSetHyperlink("ADVFN",url,1,2)
	
	
	mainSheet.getCellByPosition(2,1).String = wwwAddress
	functionSetHyperlink(wwwAddress, wwwAddress,2,1)
	
	if latestPrice: mainSheet.getCellByPosition(1,latestPriceRow-1).Value = latestPrice
	if bidPrice: mainSheet.getCellByPosition(1,bidPriceRow-1).Value = bidPrice
	if offerPrice: mainSheet.getCellByPosition(1,offerPriceRow-1).Value = offerPrice
	if marketCap: mainSheet.getCellByPosition(1,marketCapRow-1).Value = marketCap
	if sharesInIssue: mainSheet.getCellByPosition(1,sharesInIssueRow-1).Value = sharesInIssue
	if pe: mainSheet.getCellByPosition(1,peRow-1).Value = pe
	if dividendYield: mainSheet.getCellByPosition(1,dividendYieldRow-1).Value = dividendYield
	if dividendCover: mainSheet.getCellByPosition(1,dividendCoverRow-1).Value = dividendCover
	if grossGearing: mainSheet.getCellByPosition(1,grossGearingRow-1).Value = grossGearing
	if divYrp1: 
		if divSpecYrp1:
			mainSheet.getCellByPosition(1,divYrp1Row-1).String = divCurYrp1 + ' ' + str(divYrp1) + '*'
		else:
			mainSheet.getCellByPosition(1,divYrp1Row-1).String = divCurYrp1 + ' ' + str(divYrp1)
	if divYr0: 
		if divSpecYr0:
			mainSheet.getCellByPosition(1,divYr0Row-1).String = divCurYr0 + ' ' + str(divYr0) + '*'
		else:
			mainSheet.getCellByPosition(1,divYr0Row-1).String = divCurYr0 + ' ' + str(divYr0)
	if divYrm1: 
		if divSpecYrm1:
			mainSheet.getCellByPosition(1,divYrm1Row-1).String = divCurYrm1 + ' ' + str(divYrm1) + '*'
		else:
			mainSheet.getCellByPosition(1,divYrm1Row-1).String = divCurYrm1 + ' ' + str(divYrm1)
	if divYrm2: 
		if divSpecYrm2:
			mainSheet.getCellByPosition(1,divYrm2Row-1).String = divCurYrm2 + ' ' + str(divYrm2) + '*'
		else:
			mainSheet.getCellByPosition(1,divYrm2Row-1).String = divCurYrm2 + ' ' + str(divYrm2)
	if divYrm3: 
		if divSpecYrm3:
			mainSheet.getCellByPosition(1,divYrm3Row-1).String = divCurYrm3 + ' ' + str(divYrm3) + '*'
		else:
			mainSheet.getCellByPosition(1,divYrm3Row-1).String = divCurYrm3 + ' ' + str(divYrm3)
	if divYrm4: 
		if divSpecYrm4:
			mainSheet.getCellByPosition(1,divYrm4Row-1).String = divCurYrm4 + ' ' + str(divYrm4) + '*'
		else:
			mainSheet.getCellByPosition(1,divYrm4Row-1).String = divCurYrm4 + ' ' + str(divYrm4)

	# get data from MoneyAM
	oStatusIndicator.setText("Processing MoneyAM")
	oStatusIndicator.setValue(1)
	resetValues()
	function_2()

	functionSetHyperlink("MoneyAM", url,2,2)
	
	if latestPrice: mainSheet.getCellByPosition(2,latestPriceRow-1).Value = latestPrice
	if bidPrice: mainSheet.getCellByPosition(2,bidPriceRow-1).Value = bidPrice
	if offerPrice: mainSheet.getCellByPosition(2,offerPriceRow-1).Value = offerPrice
	if marketCap: mainSheet.getCellByPosition(2,marketCapRow-1).Value = marketCap
	if sharesInIssue: mainSheet.getCellByPosition(2,sharesInIssueRow-1).Value = sharesInIssue
	if pe: mainSheet.getCellByPosition(2,peRow-1).Value = pe
	if dividendYield: mainSheet.getCellByPosition(2,dividendYieldRow-1).Value = dividendYield
	if dividendCover: mainSheet.getCellByPosition(2,dividendCoverRow-1).Value = dividendCover
	if grossGearing: mainSheet.getCellByPosition(2,grossGearingRow-1).Value = grossGearing
	if divYrp1: 
		if divSpecYrp1:
			mainSheet.getCellByPosition(2,divYrp1Row-1).String = divCurYrp1 + ' ' + str(divYrp1) + '*'
		else:
			mainSheet.getCellByPosition(2,divYrp1Row-1).String = divCurYrp1 + ' ' + str(divYrp1)
	if divYr0: 
		if divSpecYr0:
			mainSheet.getCellByPosition(2,divYr0Row-1).String = divCurYr0 + ' ' + str(divYr0) + '*'
		else:
			mainSheet.getCellByPosition(2,divYr0Row-1).String = divCurYr0 + ' ' + str(divYr0)
	if divYrm1: 
		if divSpecYrm1:
			mainSheet.getCellByPosition(2,divYrm1Row-1).String = divCurYrm1 + ' ' + str(divYrm1) + '*'
		else:
			mainSheet.getCellByPosition(2,divYrm1Row-1).String = divCurYrm1 + ' ' + str(divYrm1)
	if divYrm2: 
		if divSpecYrm2:
			mainSheet.getCellByPosition(2,divYrm2Row-1).String = divCurYrm2 + ' ' + str(divYrm2) + '*'
		else:
			mainSheet.getCellByPosition(2,divYrm2Row-1).String = divCurYrm2 + ' ' + str(divYrm2)
	if divYrm3: 
		if divSpecYrm3:
			mainSheet.getCellByPosition(2,divYrm3Row-1).String = divCurYrm3 + ' ' + str(divYrm3) + '*'
		else:
			mainSheet.getCellByPosition(2,divYrm3Row-1).String = divCurYrm3 + ' ' + str(divYrm3)
	if divYrm4: 
		if divSpecYrm4:
			mainSheet.getCellByPosition(2,divYrm4Row-1).String = divCurYrm4 + ' ' + str(divYrm4) + '*'
		else:
			mainSheet.getCellByPosition(2,divYrm4Row-1).String = divCurYrm4 + ' ' + str(divYrm4)
	
	

	# get data from Yahoo
	oStatusIndicator.setText("Processing Yahoo")
	oStatusIndicator.setValue(2)
	resetValues()
	if yahooSymbol: 
		function_3()
		functionSetHyperlink("Yahoo",url,3,2)

	if marketCap: mainSheet.getCellByPosition(3,marketCapRow-1).Value = marketCap
	if latestPrice: mainSheet.getCellByPosition(3,latestPriceRow-1).Value = latestPrice
	if bidPrice: mainSheet.getCellByPosition(3,bidPriceRow-1).Value = bidPrice
	if offerPrice: mainSheet.getCellByPosition(3,offerPriceRow-1).Value = offerPrice
	

	# get data from morningstar
	oStatusIndicator.setText("Processing Morningstar")
	oStatusIndicator.setValue(3)
	resetValues()
	if mStarId: 
		function_4()
		functionSetHyperlink("MorningStar",url,4,2)
		
	if latestPrice: mainSheet.getCellByPosition(4,latestPriceRow-1).Value = latestPrice
	if bidPrice: mainSheet.getCellByPosition(4,bidPriceRow-1).Value = bidPrice
	if offerPrice: mainSheet.getCellByPosition(4,offerPriceRow-1).Value = offerPrice
	if marketCap: mainSheet.getCellByPosition(4,marketCapRow-1).Value = marketCap
	if sharesInIssue: mainSheet.getCellByPosition(4,sharesInIssueRow-1).Value = sharesInIssue
	if pe: mainSheet.getCellByPosition(4,peRow-1).Value = pe
	if dividendYield: mainSheet.getCellByPosition(4,dividendYieldRow-1).Value = dividendYield
	if dividendCover: mainSheet.getCellByPosition(4,dividendCoverRow-1).Value = dividendCover
	if grossGearing: mainSheet.getCellByPosition(4,grossGearingRow-1).Value = grossGearing
	if divFcstYrp2: mainSheet.getCellByPosition(4,divFcstYrp2Row-1).String = "GBP " + str(divFcstYrp2)
	if divFcstYrp1: mainSheet.getCellByPosition(4,divFcstYrp1Row-1).String = "GBP " + str(divFcstYrp1)
	if divFcstYr0: mainSheet.getCellByPosition(4,divFcstYr0Row-1).String = "GBP " + str(divFcstYr0)
	
	
	if mStarId: function_4a()
	if divYr0: mainSheet.getCellByPosition(4,divYr0Row-1).String = 'GBP ' + str(divYr0)
	if divYrm1: mainSheet.getCellByPosition(4,divYrm1Row-1).String = 'GBP ' + str(divYrm1)
	if divYrm2: mainSheet.getCellByPosition(4,divYrm2Row-1).String = 'GBP ' + str(divYrm2)
	if divYrm3: mainSheet.getCellByPosition(4,divYrm3Row-1).String = 'GBP ' + str(divYrm3)
	if divYrm4: mainSheet.getCellByPosition(4,divYrm4Row-1).String = 'GBP ' + str(divYrm4)
	
	
	# get data from MSN Money
	oStatusIndicator.setText("Processing MSN Money")
	oStatusIndicator.setValue(4)
	resetValues()
	function_5()
	
	functionSetHyperlink("MSN Money",url,5,2)
	if latestPrice: mainSheet.getCellByPosition(5,latestPriceRow-1).Value = latestPrice
	if bidPrice: mainSheet.getCellByPosition(5,bidPriceRow-1).Value = bidPrice
	if offerPrice: mainSheet.getCellByPosition(5,offerPriceRow-1).Value = offerPrice
	if marketCap:mainSheet.getCellByPosition(5,marketCapRow-1).Value = marketCap
	if sharesInIssue: mainSheet.getCellByPosition(5,sharesInIssueRow-1).Value = sharesInIssue
	if pe: mainSheet.getCellByPosition(5,peRow-1).Value = pe
	if dividendYield: mainSheet.getCellByPosition(5,dividendYieldRow-1).Value = dividendYield
	if dividendCover: mainSheet.getCellByPosition(5,dividendCoverRow-1).Value = dividendCover
	if grossGearing: mainSheet.getCellByPosition(5,grossGearingRow-1).Value = grossGearing
	
	
	
	# get data from dividend investor
	oStatusIndicator.setText("Processing Dividend Investor")
	oStatusIndicator.setValue(5)
	resetValues()
	function_6a()
	function_6()

	functionSetHyperlink("Dividend Investor",url,6,2)
	if latestPrice: 
		if latestPrice.endswith('$'):
			latestPrice = latestPrice.replace('$','')
			mainSheet.getCellByPosition(6,latestPriceRow-1).String = 'USD ' + latestPrice
		elif latestPrice.endswith('p'):
			latestPrice = latestPrice.replace('p','')
			mainSheet.getCellByPosition(6,latestPriceRow-1).Value = latestPrice
		else:
			mainSheet.getCellByPosition(6,latestPriceRow-1).String = latestPrice
	if marketCap: mainSheet.getCellByPosition(6,marketCapRow-1).Value = marketCap
	if sharesInIssue: mainSheet.getCellByPosition(6,sharesInIssueRow-1).Value = sharesInIssue
	if dividendYield: mainSheet.getCellByPosition(6,dividendYieldRow-1).Value = dividendYield
	if divYrp1: 
		if divSpecYrp1:
			mainSheet.getCellByPosition(6,divYrp1Row-1).String = divCur + ' ' + str(divYrp1) + '*'
		else:
			mainSheet.getCellByPosition(6,divYrp1Row-1).String = divCur + ' ' + str(divYrp1)
	if divYr0: 
		if divSpecYr0:
			mainSheet.getCellByPosition(6,divYr0Row-1).String = divCur + ' ' + str(divYr0) + '*'
		else:
			mainSheet.getCellByPosition(6,divYr0Row-1).String = divCur + ' ' + str(divYr0)
	if divYrm1: 
		if divSpecYrm1:
			mainSheet.getCellByPosition(6,divYrm1Row-1).String = divCur + ' ' + str(divYrm1) + '*'
		else:
			mainSheet.getCellByPosition(6,divYrm1Row-1).String = divCur + ' ' + str(divYrm1)
	if divYrm2: 
		if divSpecYrm2:
			mainSheet.getCellByPosition(6,divYrm2Row-1).String = divCur + ' ' + str(divYrm2) + '*'
		else:
			mainSheet.getCellByPosition(6,divYrm2Row-1).String = divCur + ' ' + str(divYrm2)
	if divYrm3: 
		if divSpecYrm3:
			mainSheet.getCellByPosition(6,divYrm3Row-1).String = divCur + ' ' + str(divYrm3) + '*'
		else:
			mainSheet.getCellByPosition(6,divYrm3Row-1).String = divCur + ' ' + str(divYrm3)
	if divYrm4: 
		if divSpecYrm4:
			mainSheet.getCellByPosition(6,divYrm4Row-1).String = divCur + ' ' + str(divYrm4) + '*'
		else:
			mainSheet.getCellByPosition(6,divYrm4Row-1).String = divCur + ' ' + str(divYrm4)
	
	oStatusIndicator.setValue(6)
	
	# get price data from Digital Look
	oStatusIndicator.setText("Processing Digital Look")
	oStatusIndicator.setValue(6)
	resetValues()
	if dlCSI: 
		function_7a()
		functionSetHyperlink("Digital Look",url,7,2)
		
	if latestPrice: 
		if latestPrice.endswith('p'):
			latestPrice = latestPrice.replace('p','')
			mainSheet.getCellByPosition(7,latestPriceRow-1).Value = latestPrice
		else:
			mainSheet.getCellByPosition(7,latestPriceRow-1).String = latestPrice
	if bidPrice: 
		if bidPrice.endswith('p'):
			bidPrice = bidPrice.replace('p','')
			mainSheet.getCellByPosition(7,bidPriceRow-1).Value = bidPrice
		else:
			mainSheet.getCellByPosition(7,bidPriceRow-1).String = bidPrice
	if offerPrice: 
		if offerPrice.endswith('p'):
			offerPrice = offerPrice.replace('p','')
			mainSheet.getCellByPosition(7,offerPriceRow-1).Value = offerPrice
		else:
			mainSheet.getCellByPosition(7,offerPriceRow-1).String = offerPrice
	if marketCap: mainSheet.getCellByPosition(7,marketCapRow-1).Value = marketCap
	if sharesInIssue: mainSheet.getCellByPosition(7,sharesInIssueRow-1).Value = sharesInIssue
	
	# get PE, yield, div cover and forecast div from digital look
	if dlCSI: function_7b()

	if pe: mainSheet.getCellByPosition(7,peRow-1).Value = pe
	if dividendYield: mainSheet.getCellByPosition(7,dividendYieldRow-1).Value = dividendYield
	if dividendCover: mainSheet.getCellByPosition(7,dividendCoverRow-1).Value = dividendCover

	if divFcstYr0: 
		if divFcstYr0.endswith('p'):
			divFcstYr0 = divFcstYr0.replace('p','')
			mainSheet.getCellByPosition(7,divFcstYr0Row-1).String = 'GBP ' + divFcstYr0
		else:
			mainSheet.getCellByPosition(7,divFcstYr0Row-1).String = divFcstYr0
	if divFcstYrp1: 
		if divFcstYrp1.endswith('p'):
			divFcstYrp1 = divFcstYrp1.replace('p','')
			mainSheet.getCellByPosition(7,divFcstYrp1Row-1).String = 'GBP ' + divFcstYrp1
		else:
			mainSheet.getCellByPosition(7,divFcstYrp1Row-1).String = divFcstYrp1
	if divFcstYrp2: 
		if divFcstYrp2.endswith('p'):
			divFcstYrp2 = divFcstYrp2.replace('p','')
			mainSheet.getCellByPosition(7,divFcstYrp2Row-1).String = 'GBP ' + divFcstYrp2
		else:
			mainSheet.getCellByPosition(7,divFcstYrp2Row-1).String = divFcstYrp2

# get dividends data from Digital Look
	oStatusIndicator.setText("Processing Digital Look dividends")
	oStatusIndicator.setValue(6)
	mainCurrencyTemp = mainCurrency
	resetValues()
	if dlCSI: 
		function_7c()
		
		
		if divYrp1: 
			if divCurYrp1 == 'p':
				divCurYrp1 = 'GBP'
			elif divCurYrp1 == 'c' and mainCurrencyTemp == '$':
				divCurYrp1 = 'USD'
			else:
				divCurYrp1 = 'EUR'
			
			divYrp1 = round(divYrp1,5)
			if divSpecYrp1:
				mainSheet.getCellByPosition(7,divYrp1Row-1).String = divCurYrp1 + ' ' + str(divYrp1) + '*'
			else:
				mainSheet.getCellByPosition(7,divYrp1Row-1).String = divCurYrp1 + ' ' + str(divYrp1)
		if divYr0: 
			if divCurYr0 == 'p':
				divCurYr0 = 'GBP'
			elif divCurYr0 == 'c' and mainCurrencyTemp == '$':
				divCurYr0 = 'USD'
			else:
				divCurYr0 = 'EUR'
			
			divYr0 = round(divYr0,5)
			if divSpecYr0:
				mainSheet.getCellByPosition(7,divYr0Row-1).String = divCurYr0 + ' ' + str(divYr0) + '*'
			else:
				mainSheet.getCellByPosition(7,divYr0Row-1).String = divCurYr0 + ' ' + str(divYr0)
		if divYrm1: 
			if divCurYrm1 == 'p':
				divCurYrm1 = 'GBP'
			elif divCurYrm1 == 'c' and mainCurrencyTemp == '$':
				divCurYrm1 = 'USD'
			else:
				divCurYrm1 = 'EUR'
			
			divYrm1 = round(divYrm1,5)
			if divSpecYrm1:
				mainSheet.getCellByPosition(7,divYrm1Row-1).String = divCurYrm1 + ' ' + str(divYrm1) + '*'
			else:
				mainSheet.getCellByPosition(7,divYrm1Row-1).String = divCurYrm1 + ' ' + str(divYrm1)
		if divYrm2: 
			if divCurYrm2 == 'p':
				divCurYrm2 = 'GBP'
			elif divCurYrm2 == 'c' and mainCurrencyTemp == '$':
				divCurYrm2 = 'USD'
			else:
				divCurYrm2 = 'EUR'
			
			divYrm2 = round(divYrm2,5)
			if divSpecYrm2:
				mainSheet.getCellByPosition(7,divYrm2Row-1).String = divCurYrm2 + ' ' + str(divYrm2) + '*'
			else:
				mainSheet.getCellByPosition(7,divYrm2Row-1).String = divCurYrm2 + ' ' + str(divYrm2)
		if divYrm3: 
			if divCurYrm3 == 'p':
				divCurYrm3 = 'GBP'
			elif divCurYrm3 == 'c' and mainCurrencyTemp == '$':
				divCurYrm3 = 'USD'
			else:
				divCurYrm3 = 'EUR'
			
			divYrm3 = round(divYrm3,5)
			if divSpecYrm3:
				mainSheet.getCellByPosition(7,divYrm3Row-1).String = divCurYrm3 + ' ' + str(divYrm3) + '*'
			else:
				mainSheet.getCellByPosition(7,divYrm3Row-1).String = divCurYrm3 + ' ' + str(divYrm3)
		if divYrm4: 
			if divCurYrm4 == 'p':
				divCurYrm4 = 'GBP'
			elif divCurYrm4 == 'c' and mainCurrencyTemp == '$':
				divCurYrm4 = 'USD'
			else:
				divCurYrm4 = 'EUR'
			
			divYrm4 = round(divYrm4,5)
			if divSpecYrm4:
				mainSheet.getCellByPosition(7,divYrm4Row-1).String = divCurYrm4 + ' ' + str(divYrm4) + '*'
			else:
				mainSheet.getCellByPosition(7,divYrm4Row-1).String = divCurYrm4 + ' ' + str(divYrm4)
		
			
			
# get price data from Interactive Investor
	oStatusIndicator.setText("Processing iii")
	oStatusIndicator.setValue(7)
	resetValues()
	if iiiSymbol:
		function_8()
		
	if latestPrice: mainSheet.getCellByPosition(8,latestPriceRow-1).Value = latestPrice
	if bidPrice: mainSheet.getCellByPosition(8,bidPriceRow-1).Value = bidPrice
	if offerPrice: mainSheet.getCellByPosition(8,offerPriceRow-1).Value = offerPrice
	
	if iiiSymbol: 
		function_8a()
		functionSetHyperlink("Interactive Investor",url,8,2)
	
	if marketCap: mainSheet.getCellByPosition(8,marketCapRow-1).Value = marketCap
	if sharesInIssue: mainSheet.getCellByPosition(8,sharesInIssueRow-1).Value = sharesInIssue
	if pe: mainSheet.getCellByPosition(8,peRow-1).Value = pe
	if dividendYield: mainSheet.getCellByPosition(8,dividendYieldRow-1).Value = dividendYield
	if dividendCover: mainSheet.getCellByPosition(8,dividendCoverRow-1).Value = dividendCover
	
	if divYrp1: 
		if divSpecYrp1:
			mainSheet.getCellByPosition(8,divYrp1Row-1).String = divCurYrp1 + ' ' + str(divYrp1) + '*'
		else:
			mainSheet.getCellByPosition(8,divYrp1Row-1).String = divCurYrp1 + ' ' + str(divYrp1)
	if divYr0: 
		if divSpecYr0:
			mainSheet.getCellByPosition(8,divYr0Row-1).String = divCurYr0 + ' ' + str(divYr0) + '*'
		else:
			mainSheet.getCellByPosition(8,divYr0Row-1).String = divCurYr0 + ' ' + str(divYr0)
	if divYrm1: 
		if divSpecYrm1:
			mainSheet.getCellByPosition(8,divYrm1Row-1).String = divCurYrm1 + ' ' + str(divYrm1) + '*'
		else:
			mainSheet.getCellByPosition(8,divYrm1Row-1).String = divCurYrm1 + ' ' + str(divYrm1)
	if divYrm2: 
		if divSpecYrm2:
			mainSheet.getCellByPosition(8,divYrm2Row-1).String = divCurYrm2 + ' ' + str(divYrm2) + '*'
		else:
			mainSheet.getCellByPosition(8,divYrm2Row-1).String = divCurYrm2 + ' ' + str(divYrm2)
	if divYrm3: 
		if divSpecYrm3:
			mainSheet.getCellByPosition(8,divYrm3Row-1).String = divCurYrm3 + ' ' + str(divYrm3) + '*'
		else:
			mainSheet.getCellByPosition(8,divYrm3Row-1).String = divCurYrm3 + ' ' + str(divYrm3)
	if divYrm4: 
		if divSpecYrm4:
			mainSheet.getCellByPosition(8,divYrm4Row-1).String = divCurYrm4 + ' ' + str(divYrm4) + '*'
		else:
			mainSheet.getCellByPosition(8,divYrm4Row-1).String = divCurYrm4 + ' ' + str(divYrm4)
	
	
	
	oStatusIndicator.setText("")
	oStatusIndicator.end()	# restore status bar
	messageBox("Process Finished", "Success")